from capybaradb._collection import Collection

class Database:
    def __init__(self, api_key: str, project_id: str, db_name: str):
        self.api_key = api_key
        self.project_id = project_id
        self.db_name = db_name

    def collection(self, collection_name: str) -> Collection:
        """
        Get a collection instance within this database.
        :param collection_name: The name of the collection
        :return: Collection instance
        """
        return Collection(self.api_key, self.project_id, self.db_name, collection_name)

    def __getattr__(self, name: str) -> Collection:
        """
        Dynamically return a 'Collection' object when accessing as an attribute.
        :param name: The name of the collection
        :return: Collection instance
        """
        return self.collection(name)

    def __getitem__(self, name: str) -> Collection:
        """
        Dynamically return a 'Collection' object when accessing via dictionary syntax.
        :param name: The name of the collection
        :return: Collection instance
        """
        return self.collection(name)



from typing import Optional, Dict


class EmbText:
    SUPPORTED_EMB_MODELS = [
        "text-embedding-3-small",
        "text-embedding-3-large",
        "ada v2",
    ]

    def __init__(self, text: str, emb_model: Optional[str] = None):
        if not self.is_valid_text(text):
            raise ValueError("Invalid text: must be a non-empty string.")
        if emb_model and not self.is_valid_emb_model(emb_model):
            raise ValueError(f"Invalid embedding model: {emb_model} is not supported.")

        self.text = text
        self.emb_model = emb_model

    @staticmethod
    def is_valid_text(text: str) -> bool:
        return isinstance(text, str) and text.strip() != ""

    @classmethod
    def is_valid_emb_model(cls, emb_model: str) -> bool:
        return emb_model in cls.SUPPORTED_EMB_MODELS

    def to_json(self) -> Dict[str, Dict[str, str]]:
        """
        Convert the EmbText instance to a JSON-serializable dictionary.
        """
        data = {"@embText": {"text": self.text}}
        if self.emb_model:
            data["@embText"]["emb_model"] = self.emb_model
        return data

    @classmethod
    def from_json(cls, data: Dict[str, Dict[str, str]]) -> "EmbText":
        """
        Create an EmbText instance from a JSON-serializable dictionary.
        """
        emb_text_data = data.get("@embText", {})
        text = emb_text_data.get("text")
        emb_model = emb_text_data.get("emb_model")
        return cls(text, emb_model)



import requests
from ._emb_json._emb_text import EmbText


class APIClientError(Exception):
    """Base class for all API client-related errors."""

    def __init__(self, status_code, message):
        super().__init__(message)
        self.status_code = status_code
        self.message = message


class AuthenticationError(APIClientError):
    """Error raised for authentication-related issues."""

    pass


class ClientRequestError(APIClientError):
    """Error raised for client-side issues such as validation errors."""

    pass


class ServerError(APIClientError):
    """Error raised for server-side issues."""

    pass


class Collection:
    def __init__(
        self, api_key: str, project_id: str, db_name: str, collection_name: str
    ):
        self.api_key = api_key
        self.project_id = project_id
        self.db_name = db_name
        self.collection_name = collection_name

    def get_collection_url(self) -> str:
        return f"https://api.capybaradb.co/v0/db/{self.project_id}_{self.db_name}/collection/{self.collection_name}/document"

    def get_headers(self) -> dict:
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

    def transform_emb_text(self, document: dict) -> dict:
        """
        Recursively traverse the document and convert EmbText instances to JSON.
        """
        for key, value in document.items():
            if isinstance(value, EmbText):
                document[key] = value.to_json()  # Convert EmbText to JSON
            elif isinstance(value, dict):
                document[key] = self.transform_emb_text(
                    value
                )  # Recurse for nested dicts
            elif isinstance(value, list):
                document[key] = [
                    self.transform_emb_text(item) if isinstance(item, dict) else item
                    for item in value
                ]  # Handle lists of dicts or other values
        return document

    def handle_response(self, response):
        try:
            response.raise_for_status()
            return response.json()
        except requests.exceptions.HTTPError as e:
            try:
                # Attempt to parse the JSON error response
                error_data = response.json()
                status = error_data.get("status", "error")
                code = error_data.get("code", 500)
                message = error_data.get("message", "An unknown error occurred.")

                if code == 401:
                    raise AuthenticationError(code, message) from e
                elif code >= 400 and code < 500:
                    raise ClientRequestError(code, message) from e
                else:
                    raise ServerError(code, message) from e

            except ValueError:
                # Response is not JSON; use the raw text
                raise APIClientError(response.status_code, response.text) from e

    def insert(self, documents: list[dict]) -> dict:
        url = self.get_collection_url()
        headers = self.get_headers()
        transformed_documents = [self.transform_emb_text(doc) for doc in documents]
        data = {"documents": transformed_documents}

        response = requests.post(url, headers=headers, json=data)
        return self.handle_response(response)

    def update(self, filter: dict, update: dict, upsert: bool = False) -> dict:
        url = self.get_collection_url()
        headers = self.get_headers()
        transformed_update = self.transform_emb_text(update)
        data = {"filter": filter, "update": transformed_update, "upsert": upsert}

        response = requests.put(url, headers=headers, json=data)
        return self.handle_response(response)

    def delete(self, filter: dict) -> dict:
        url = self.get_collection_url()
        headers = self.get_headers()
        data = {"filter": filter}

        response = requests.delete(url, headers=headers, json=data)
        return self.handle_response(response)

    def find(
        self,
        filter: dict,
        projection: dict = None,
        sort: dict = None,
        limit: int = None,
        skip: int = None,
    ) -> list[dict]:
        url = f"{self.get_collection_url()}/find"
        headers = self.get_headers()
        data = {
            "filter": filter,
            "projection": projection,
            "sort": sort,
            "limit": limit,
            "skip": skip,
        }

        response = requests.post(url, headers=headers, json=data)
        return self.handle_response(response)

    def query(
        self,
        query: str,
        emb_model: str = None,
        top_k: int = None,
        include_values: bool = None,
        projection: dict = None,
    ) -> list[dict]:
        url = f"{self.get_collection_url()}/query"
        headers = self.get_headers()

        # Create the data dictionary with only non-None values
        data = {"query": query}  # 'query' is required
        if emb_model is not None:
            data["emb_model"] = emb_model
        if top_k is not None:
            data["top_k"] = top_k
        if include_values is not None:
            data["include_values"] = include_values
        if projection is not None:
            data["projection"] = projection

        response = requests.post(url, headers=headers, json=data)
        return self.handle_response(response)



import os
import requests
from capybaradb._database import Database


class CapybaraDB:
    def __init__(self):
        # Ensure that environment variables are checked and valid
        self.project_id = os.getenv("CAPYBARA_PROJECT_ID", "")
        self.api_key = os.getenv("CAPYBARA_API_KEY", "")

        # Validate that both values are provided
        if not self.project_id:
            raise ValueError(
                "Missing Project ID: Please provide the Project ID as an argument or set it in the CAPYBARA_PROJECT_ID environment variable. "
                "Tip: Ensure your environment file (e.g., .env) is loaded."
            )

        if not self.api_key:
            raise ValueError(
                "Missing API Key: Please provide the API Key as an argument or set it in the CAPYBARA_API_KEY environment variable. "
                "Tip: Ensure your environment file (e.g., .env) is loaded."
            )

        self.base_url = f"https://api.capybaradb.co/{self.project_id}".rstrip("/")
        self.session = requests.Session()
        self.session.headers.update({"Authorifsdzation": f"Bearer {self.api_key}"})

    def db(self, db_name: str) -> Database:
        """
        Get a database instance.
        :param db_name: The name of the database
        :return: Database instance
        """
        return Database(self.api_key, self.project_id, db_name)

    def __getattr__(self, name):
        """
        Dynamically return a 'Database' object when accessing as an attribute.
        :param name: The name of the database
        :return: Database instance
        """
        return self.db(name)

    def __getitem__(self, name):
        """
        Dynamically return a 'Database' object when accessing via dictionary syntax.
        :param name: The name of the database
        :return: Database instance
        """
        return self.db(name)
